/*
 * Copyright [2013-2021], Alibaba Group Holding Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.polardbx.executor.mpp.operator.factory;

import com.alibaba.polardbx.executor.operator.Executor;
import com.alibaba.polardbx.executor.operator.ValueExec;
import com.alibaba.polardbx.executor.utils.ExecUtils;
import com.alibaba.polardbx.optimizer.context.ExecutionContext;
import com.alibaba.polardbx.optimizer.utils.CalciteUtils;
import com.alibaba.polardbx.statistics.RuntimeStatHelper;
import org.apache.calcite.rel.logical.LogicalValues;

public class ValueExecutorFactory extends ExecutorFactory {

    private LogicalValues logicalValues;

    public ValueExecutorFactory(LogicalValues logicalValues) {
        this.logicalValues = logicalValues;
    }

    @Override
    public Executor createExecutor(ExecutionContext context, int index) {
        int count = ExecUtils.getTupleCount(logicalValues, context);
        Executor exec = new ValueExec(count, CalciteUtils.getTypes(logicalValues.getRowType()), context);
        exec.setId(logicalValues.getRelatedId());
        if (context.getRuntimeStatistics() != null) {
            RuntimeStatHelper.registerStatForExec(logicalValues, exec, context);
        }
        return exec;
    }
}
