/*
 * Copyright [2013-2021], Alibaba Group Holding Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.polardbx.optimizer.core.rel;

import com.alibaba.polardbx.common.jdbc.BytesSql;
import com.alibaba.polardbx.optimizer.context.ExecutionContext;
import org.apache.calcite.rel.RelNode;
import org.apache.calcite.rel.type.RelDataType;

import java.util.List;

public class TableOperationFactory extends PhyOperationBuilderCommon {

    private static final TableOperationFactory instance = new TableOperationFactory();

    private TableOperationFactory() {
    }

    public static TableOperationFactory getInstance() {
        return instance;
    }

    public DirectTableOperation buildDirectTableOperation(RelNode logicalPlan,
                                                          RelDataType rowType,
                                                          List<String> logicalTableNames,
                                                          List<String> tableNames,
                                                          String dbIndex,
                                                          BytesSql sqlTemplate,
                                                          List<Integer> paramIndex) {
        return new DirectTableOperation(logicalPlan, rowType, logicalTableNames, tableNames, dbIndex, sqlTemplate,
            paramIndex);
    }

    public DirectShardingKeyTableOperation buildDirectShardingKeyTableOperation(LogicalView logicalPlan,
                                                                                RelDataType rowType,
                                                                                String logTableName,
                                                                                BytesSql bytesSql,
                                                                                List<Integer> paramIndex,
                                                                                ExecutionContext ec) {
        return new DirectShardingKeyTableOperation(logicalPlan, rowType, logTableName, bytesSql, paramIndex, ec);
    }

    public SingleTableOperation buildSingleTableOperation(RelNode logicalPlan,
                                                          ShardProcessor shardProcessor,
                                                          String logicalTableName,
                                                          BytesSql sqlTemplate,
                                                          List<Integer> paramIndex,
                                                          int autoIncParamIndex) {
        return new SingleTableOperation(logicalPlan, shardProcessor, logicalTableName, sqlTemplate, paramIndex,
            autoIncParamIndex);
    }

    public SingleTableInsert buildSingleTableInsert(RelNode logicalPlan,
                                                    ShardProcessor shardProcessor,
                                                    String logicalTableName,
                                                    BytesSql sqlTemplate,
                                                    List<Integer> paramIndex,
                                                    int autoIncParamIndex) {
        return new SingleTableInsert(logicalPlan, shardProcessor, logicalTableName, sqlTemplate, paramIndex,
            autoIncParamIndex);
    }

}
