/*
 * Copyright 1999-2017 Alibaba Group Holding Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alibaba.polardbx.druid.sql.dialect.mysql.ast.statement;

import com.alibaba.polardbx.druid.sql.ast.SQLExpr;
import com.alibaba.polardbx.druid.sql.ast.SQLName;
import com.alibaba.polardbx.druid.sql.ast.expr.SQLCharExpr;
import com.alibaba.polardbx.druid.sql.ast.expr.SQLIdentifierExpr;
import com.alibaba.polardbx.druid.sql.ast.statement.SQLCreateStatement;
import com.alibaba.polardbx.druid.sql.dialect.mysql.ast.MySqlObjectImpl;
import com.alibaba.polardbx.druid.sql.dialect.mysql.ast.expr.MySqlUserName;
import com.alibaba.polardbx.druid.sql.dialect.mysql.visitor.MySqlASTVisitor;

import java.util.ArrayList;
import java.util.List;

public class MySqlCreateUserStatement extends MySqlStatementImpl implements SQLCreateStatement {

    private List<UserSpecification> users = new ArrayList<UserSpecification>(2);

    private boolean ifNotExists = false;

    public List<UserSpecification> getUsers() {
        return users;
    }

    public boolean isIfNotExists() {
        return ifNotExists;
    }

    public void setIfNotExists(boolean ifNotExists) {
        this.ifNotExists = ifNotExists;
    }

    public void addUser(UserSpecification user) {
        if (user != null) {
            user.setParent(this);
        }
        this.users.add(user);
    }

    @Override
    public void accept0(MySqlASTVisitor visitor) {
        if (visitor.visit(this)) {
            acceptChild(visitor, users);
        }
        visitor.endVisit(this);
    }

    public static class UserSpecification extends MySqlObjectImpl {

        private SQLExpr user;
        private boolean passwordHash = false;
        private SQLExpr password;
        private SQLExpr authPlugin;
        private boolean pluginAs;

        public SQLExpr getUser() {
            return user;
        }

        public void setUser(SQLExpr user) {
            this.user = (SQLName) user;
        }

        public boolean isPasswordHash() {
            return passwordHash;
        }

        public void setPasswordHash(boolean passwordHash) {
            this.passwordHash = passwordHash;
        }

        public SQLExpr getPassword() {
            return password;
        }

        public void setPassword(SQLExpr password) {
            this.password = password;
        }

        public SQLExpr getAuthPlugin() {
            return authPlugin;
        }

        public void setAuthPlugin(SQLExpr authPlugin) {
            this.authPlugin = authPlugin;
        }

        public boolean isPluginAs() {
            return pluginAs;
        }

        public void setPluginAs(boolean pluginAs) {
            this.pluginAs = pluginAs;
        }

        @Override
        public void accept0(MySqlASTVisitor visitor) {
            if (visitor.visit(this)) {
                acceptChild(visitor, user);
                acceptChild(visitor, password);
                acceptChild(visitor, authPlugin);
            }
            visitor.endVisit(this);
        }

        public String getIdentifiedBy() {
            if (user instanceof MySqlUserName && ((MySqlUserName) user).getIdentifiedBy() != null) {
                return ((MySqlUserName) user).getIdentifiedBy();
            }
            if (password instanceof SQLCharExpr) {
                return ((SQLCharExpr) password).getText();
            } else {
                return null;
            }
        }
    }
}
