/*
 * Copyright 1999-2017 Alibaba Group Holding Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alibaba.polardbx.druid.bvt.sql.mysql.createTable;

import com.alibaba.polardbx.druid.sql.MysqlTest;
import com.alibaba.polardbx.druid.sql.SQLUtils;
import com.alibaba.polardbx.druid.sql.ast.SQLStatement;
import com.alibaba.polardbx.druid.sql.dialect.mysql.parser.MySqlStatementParser;
import com.alibaba.polardbx.druid.sql.dialect.mysql.visitor.MySqlSchemaStatVisitor;
import com.alibaba.polardbx.druid.stat.TableStat;
import org.junit.Assert;

import java.util.List;

public class MySqlCreateTableTest44 extends MysqlTest {

    public void test_0() throws Exception {
        String sql = "CREATE TABLE lc (" + //
                     "    a INT NULL, " + //
                     "    b INT NULL" + //
                     ")" + //
                     "PARTITION BY LIST COLUMNS(a,b) (" + //
                     "    PARTITION p0 VALUES IN( (0,0), (NULL,NULL) )," + //
                     "    PARTITION p1 VALUES IN( (0,1), (0,2), (0,3), (1,1), (1,2) )," + //
                     "    PARTITION p2 VALUES IN( (1,0), (2,0), (2,1), (3,0), (3,1) )," + //
                     "    PARTITION p3 VALUES IN( (1,3), (2,2), (2,3), (3,2), (3,3) )" + //
                     ");"; //

        MySqlStatementParser parser = new MySqlStatementParser(sql);
        List<SQLStatement> statementList = parser.parseStatementList();
        SQLStatement stmt = statementList.get(0);
//        print(statementList);

        Assert.assertEquals(1, statementList.size());

        MySqlSchemaStatVisitor visitor = new MySqlSchemaStatVisitor();
        stmt.accept(visitor);
//
//        System.out.println("Tables : " + visitor.getTables());
//        System.out.println("fields : " + visitor.getColumns());
//        System.out.println("coditions : " + visitor.getConditions());
//        System.out.println("orderBy : " + visitor.getOrderByColumns());

        Assert.assertEquals(1, visitor.getTables().size());
        Assert.assertEquals(2, visitor.getColumns().size());
        Assert.assertEquals(0, visitor.getConditions().size());

        Assert.assertTrue(visitor.getTables().containsKey(new TableStat.Name("lc")));

        String output = SQLUtils.toMySqlString(stmt);
        Assert.assertEquals("CREATE TABLE lc (\n" +
                "\ta INT NULL,\n" +
                "\tb INT NULL\n" +
                ")\n" +
                "PARTITION BY LIST COLUMNS (a, b) (\n" +
                "\tPARTITION p0 VALUES IN ((0, 0), (NULL, NULL)), \n" +
                "\tPARTITION p1 VALUES IN ((0, 1), (0, 2), (0, 3), (1, 1), (1, 2)), \n" +
                "\tPARTITION p2 VALUES IN ((1, 0), (2, 0), (2, 1), (3, 0), (3, 1)), \n" +
                "\tPARTITION p3 VALUES IN ((1, 3), (2, 2), (2, 3), (3, 2), (3, 3))\n" +
                ");", output);

    }
}
